"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAbility = exports.makeDefaultOptions = exports.makeOptions = void 0;
const getDefaultModelName_1 = require("../utils/getDefaultModelName");
const makeOptions = (app, options) => {
    if (!app) {
        throw new Error("feathers-casl: You need to provide an 'app' to the channels:makeOptions function");
    }
    options = options || {};
    return Object.assign({}, defaultOptions, getAppOptions(app), options);
};
exports.makeOptions = makeOptions;
const defaultOptions = {
    activated: true,
    channelOnError: ["authenticated"],
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ability: ((app, connection, data, context) => {
        return connection.ability;
    }),
    modelName: getDefaultModelName_1.getContextPath,
    restrictFields: true,
    availableFields: (context) => {
        var _a, _b;
        const availableFields = (_b = (_a = context.service.options) === null || _a === void 0 ? void 0 : _a.casl) === null || _b === void 0 ? void 0 : _b.availableFields;
        if (!availableFields)
            return undefined;
        return (typeof availableFields === "function")
            ? availableFields(context)
            : availableFields;
    }
};
const makeDefaultOptions = (options) => {
    return Object.assign({}, defaultOptions, options);
};
exports.makeDefaultOptions = makeDefaultOptions;
const getAppOptions = (app) => {
    const caslOptions = app === null || app === void 0 ? void 0 : app.get("casl");
    return (caslOptions && caslOptions.channels)
        ? caslOptions.channels
        : {};
};
const getAbility = (app, data, connection, context, options) => {
    if (options.ability) {
        return (typeof options.ability === "function") ?
            options.ability(app, connection, data, context) :
            options.ability;
    }
    else {
        return connection.ability;
    }
};
exports.getAbility = getAbility;
