"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const isEqual_1 = __importDefault(require("lodash/isEqual"));
const pick_1 = __importDefault(require("lodash/pick"));
const isEmpty_1 = __importDefault(require("lodash/isEmpty"));
require("@feathersjs/transport-commons");
const ability_1 = require("@casl/ability");
const channels_utils_1 = require("./channels.utils");
const getModelName_1 = __importDefault(require("../utils/getModelName"));
const hasRestrictingFields_1 = __importDefault(require("../utils/hasRestrictingFields"));
const base_1 = require("@feathersjs/transport-commons/lib/channels/channel/base");
exports.default = (app, data, context, options) => {
    options = channels_utils_1.makeOptions(app, options);
    const { channelOnError, activated } = options;
    const modelName = getModelName_1.default(options.modelName, context);
    if (!activated || !modelName) {
        return (!channelOnError) ? new base_1.Channel() : app.channel(channelOnError);
    }
    const { channels } = app;
    //return app.channel(channels);
    const allConnections = app.channel(channels).connections;
    const dataToTest = ability_1.subject(modelName, data);
    if (!options.restrictFields) {
        // return all fields for allowed 
        let connections = allConnections
            .filter(connection => {
            const ability = channels_utils_1.getAbility(app, data, connection, context, options);
            return ability && ability.can("get", dataToTest);
        });
        connections = [...new Set(connections)];
        return new base_1.Channel(connections, data);
    }
    else {
        // filter by restricted Fields
        const connectionsPerFields = [];
        for (let i = 0, n = allConnections.length; i < n; i++) {
            const connection = allConnections[i];
            const { ability } = connection;
            if (!ability || !ability.can("get", dataToTest)) {
                // connection cannot read item -> don't send data
                continue;
            }
            const availableFields = (!(options === null || options === void 0 ? void 0 : options.availableFields))
                ? undefined
                : (typeof options.availableFields === "function")
                    ? options.availableFields(context)
                    : options.availableFields;
            const fields = hasRestrictingFields_1.default(ability, "get", dataToTest, { availableFields });
            // if fields is true or fields is empty array -> full restriction
            if (fields && (fields === true || fields.length === 0)) {
                continue;
            }
            const connField = connectionsPerFields.find(x => isEqual_1.default(x.fields, fields));
            if (connField) {
                if (connField.connections.indexOf(connection) !== -1) {
                    // connection is already in array -> skip
                    continue;
                }
                connField.connections.push(connection);
            }
            else {
                connectionsPerFields.push({
                    connections: [connection],
                    fields: fields
                });
            }
        }
        const channels = [];
        for (let i = 0, n = connectionsPerFields.length; i < n; i++) {
            const { fields, connections } = connectionsPerFields[i];
            const restrictedData = (fields)
                ? pick_1.default(data, fields)
                : data;
            if (!isEmpty_1.default(restrictedData)) {
                channels.push(new base_1.Channel(connections, restrictedData));
            }
        }
        return channels.length === 1
            ? channels[0]
            : channels;
    }
};
