"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const feathers_hooks_common_1 = require("feathers-hooks-common");
const ability_1 = require("@casl/ability");
const pick_1 = __importDefault(require("lodash/pick"));
const isEmpty_1 = __importDefault(require("lodash/isEmpty"));
const hasRestrictingFields_1 = __importDefault(require("../../utils/hasRestrictingFields"));
const feathers_utils_1 = require("feathers-utils");
const authorize_hook_utils_1 = require("./authorize.hook.utils");
const getModelName_1 = __importDefault(require("../../utils/getModelName"));
const errors_1 = require("@feathersjs/errors");
const HOOKNAME = "authorize";
exports.default = (options) => {
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        const $select = authorize_hook_utils_1.restore$select(context);
        if (feathers_utils_1.shouldSkip(HOOKNAME, context) ||
            context.type !== "after" ||
            !context.params) {
            return context;
        }
        options = authorize_hook_utils_1.makeOptions(context.app, options);
        const modelName = getModelName_1.default(options.modelName, context);
        if (!modelName) {
            return context;
        }
        const skipCheckConditions = authorize_hook_utils_1.getPersistedConfig(context, "skipRestrictingRead.conditions");
        const skipCheckFields = authorize_hook_utils_1.getPersistedConfig(context, "skipRestrictingRead.fields");
        if (skipCheckConditions && skipCheckFields) {
            return context;
        }
        const { params } = context;
        params.ability = yield authorize_hook_utils_1.getAbility(context, options);
        if (!params.ability) {
            // Ignore internal or not authenticated requests
            return context;
        }
        const { ability } = params;
        const items = feathers_hooks_common_1.getItems(context);
        const availableFields = (!(options === null || options === void 0 ? void 0 : options.availableFields))
            ? undefined
            : (typeof options.availableFields === "function")
                ? options.availableFields(context)
                : options.availableFields;
        const hasRestrictingFieldsOptions = {
            availableFields: availableFields
        };
        const pickFieldsForItem = (item) => {
            const method = (Array.isArray(items)) ? "find" : "get";
            if (!skipCheckConditions && !ability.can(method, ability_1.subject(modelName, item))) {
                return undefined;
            }
            let fields = hasRestrictingFields_1.default(ability, method, ability_1.subject(modelName, item), hasRestrictingFieldsOptions);
            if (fields === true) {
                // full restriction
                return {};
            }
            else if (skipCheckFields || (!fields && !$select)) {
                // no restrictions
                return item;
            }
            else if (fields && $select) {
                fields = feathers_utils_1.mergeArrays(fields, $select, "intersect");
            }
            else {
                fields = (fields) ? fields : $select;
            }
            return pick_1.default(item, fields);
        };
        let result;
        if (Array.isArray(items)) {
            result = [];
            for (let i = 0, n = items.length; i < n; i++) {
                const item = pickFieldsForItem(items[i]);
                if (item) {
                    result.push(item);
                }
            }
        }
        else {
            result = pickFieldsForItem(items);
            if (context.method === "get" && isEmpty_1.default(result)) {
                if (options.actionOnForbidden)
                    options.actionOnForbidden();
                throw new errors_1.Forbidden(`You're not allowed to ${context.method} ${modelName}`);
            }
        }
        feathers_hooks_common_1.replaceItems(context, result);
        return context;
    });
};
