"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@feathersjs/errors");
const isEmpty_1 = __importDefault(require("lodash/isEmpty"));
const pick_1 = __importDefault(require("lodash/pick"));
const ability_1 = require("@casl/ability");
const feathers_utils_1 = require("feathers-utils");
const hasRestrictingFields_1 = __importDefault(require("../../utils/hasRestrictingFields"));
const hasRestrictingConditions_1 = __importDefault(require("../../utils/hasRestrictingConditions"));
const couldHaveRestrictingFields_1 = __importDefault(require("../../utils/couldHaveRestrictingFields"));
const getConditionalQueryFor_1 = require("../../utils/getConditionalQueryFor");
const authorize_hook_utils_1 = require("./authorize.hook.utils");
const extra_1 = require("@casl/ability/extra");
const HOOKNAME = "authorize";
exports.default = (options) => {
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        //TODO: make as isomorphic hook -> for vue-client
        if (feathers_utils_1.shouldSkip(HOOKNAME, context) ||
            context.type !== "before" ||
            !context.params) {
            return context;
        }
        const { service, method, id, params } = context;
        if (!options.modelName) {
            return context;
        }
        const modelName = (typeof options.modelName === "string")
            ? options.modelName
            : options.modelName(context);
        if (!modelName) {
            return context;
        }
        const ability = yield authorize_hook_utils_1.getAbility(context, options);
        if (!ability) {
            // Ignore internal or not authenticated requests
            return context;
        }
        if (options.checkMultiActions) {
            authorize_hook_utils_1.checkMulti(context, ability, modelName, options);
        }
        authorize_hook_utils_1.throwUnlessCan(ability, method, modelName, modelName, options.actionOnForbidden);
        // if context is with multiple items, there's a change that we need to handle each item separately
        if (feathers_utils_1.isMulti(context)) {
            // if has conditions -> hide $select for after-hook, because
            if (hasRestrictingConditions_1.default(ability, "find", modelName)) {
                authorize_hook_utils_1.hide$select(context);
            }
            else {
                authorize_hook_utils_1.setPersistedConfig(context, "skipRestrictingRead.conditions", true);
            }
            // if has no restricting fields at all -> can skip _pick() in after-hook
            if (!couldHaveRestrictingFields_1.default(ability, "find", modelName)) {
                authorize_hook_utils_1.setPersistedConfig(context, "skipRestrictingRead.fields", true);
            }
        }
        const availableFields = (!(options === null || options === void 0 ? void 0 : options.availableFields))
            ? undefined
            : (typeof options.availableFields === "function")
                ? options.availableFields(context)
                : options.availableFields;
        const hasRestrictingFieldsOptions = {
            availableFields: availableFields
        };
        if (["get", "patch", "update", "remove"].includes(method) && id != null) {
            // single: get | patch | update | remove
            // get complete item for `throwUnlessCan`-check to be trustworthy
            // -> initial 'get' and 'remove' have no data at all
            // -> initial 'patch' maybe has just partial data
            // -> initial 'update' maybe has completely changed data, for what the check could pass but not for initial data
            const queryGet = Object.assign({}, params.query || {});
            delete queryGet.$select;
            const paramsGet = Object.assign({}, params, { query: queryGet });
            paramsGet.skipHooks = ((_a = params.skipHooks) === null || _a === void 0 ? void 0 : _a.slice()) || [];
            feathers_utils_1.pushSet(paramsGet, "skipHooks", `${HOOKNAME}`, { unique: true });
            const item = yield service.get(id, paramsGet);
            authorize_hook_utils_1.throwUnlessCan(ability, method, ability_1.subject(modelName, item), modelName, options.actionOnForbidden);
            if (method === "get") {
                context.result = item;
                //pushSet(context, "params.skipHooks", "after");
                return context;
            }
            // ensure that only allowed data gets changed
            if (["update", "patch"].includes(method)) {
                const fields = hasRestrictingFields_1.default(ability, method, ability_1.subject(modelName, item), hasRestrictingFieldsOptions);
                if (!fields) {
                    return context;
                }
                if (fields === true || fields.length === 0) {
                    if (options.actionOnForbidden) {
                        options.actionOnForbidden();
                    }
                    throw new errors_1.Forbidden("You're not allowed to make this request");
                }
                const data = pick_1.default(context.data, fields);
                // if fields are not overlapping -> throw
                if (isEmpty_1.default(data)) {
                    if (options.actionOnForbidden) {
                        options.actionOnForbidden();
                    }
                    throw new errors_1.Forbidden("You're not allowed to make this request");
                }
                //TODO: if some fields not match -> `actionOnForbiddenUpdate`
                if (method === "patch") {
                    context.data = data;
                }
                else {
                    // merge with initial data
                    const itemPlain = yield service._get(id, {});
                    context.data = Object.assign({}, itemPlain, data);
                }
            }
            return context;
        }
        else if (method === "find" || (["patch", "remove"].includes(method) && id == null)) {
            // multi: find | patch | remove
            if (method === "patch") {
                const fields = hasRestrictingFields_1.default(ability, method, modelName, { availableFields });
                if (fields === true) {
                    if (options.actionOnForbidden) {
                        options.actionOnForbidden();
                    }
                    throw new errors_1.Forbidden("You're not allowed to make this request");
                }
                if (fields && fields.length > 0) {
                    const data = pick_1.default(context.data, fields);
                    context.data = data;
                }
            }
            if (hasRestrictingConditions_1.default(ability, method, modelName)) {
                // TODO: if query and context.params.query differ -> separate calls
                let query;
                if ([
                    "feathers-memory",
                    "feathers-nedb",
                    "feathers-objection",
                    "feathers-sequelize"
                ]
                    .includes(options.adapter)) {
                    query = extra_1.rulesToQuery(ability, method, modelName, (rule) => {
                        const { conditions } = rule;
                        return (rule.inverted) ? { $not: conditions } : conditions;
                    });
                }
                else if (["feathers-mongoose"]
                    .includes(options.adapter)) {
                    query = extra_1.rulesToQuery(ability, method, modelName, (rule) => {
                        const { conditions } = rule;
                        return (rule.inverted) ? { $nor: [conditions] } : conditions;
                    });
                }
                else {
                    query = extra_1.rulesToQuery(ability, method, modelName, (rule) => {
                        const { conditions } = rule;
                        return (rule.inverted) ? getConditionalQueryFor_1.convertRuleToQuery(rule) : conditions;
                    });
                    if (query.$and) {
                        const { $and } = query;
                        delete query.$and;
                        $and.forEach(q => {
                            query = feathers_utils_1.mergeQuery(query, q, {
                                defaultHandle: "intersect",
                                operators: service.operators,
                                useLogicalConjunction: true
                            });
                        });
                    }
                }
                if (!isEmpty_1.default(query)) {
                    if (!context.params.query) {
                        context.params.query = query;
                    }
                    else {
                        const operators = (_b = service.options) === null || _b === void 0 ? void 0 : _b.whitelist;
                        context.params.query = feathers_utils_1.mergeQuery(context.params.query, query, {
                            defaultHandle: "intersect",
                            operators,
                            useLogicalConjunction: true
                        });
                    }
                }
            }
            return context;
        }
        else if (method === "create") {
            // create: single | multi
            // we have all information we need (maybe we need populated data?)
            const data = (Array.isArray(context.data)) ? context.data : [context.data];
            for (let i = 0; i < data.length; i++) {
                authorize_hook_utils_1.throwUnlessCan(ability, method, ability_1.subject(modelName, data[i]), modelName, options.actionOnForbidden);
            }
            return context;
        }
        return context;
    });
};
