"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertRuleToQuery = void 0;
const isPlainObject_1 = __importDefault(require("lodash/isPlainObject"));
const errors_1 = require("@feathersjs/errors");
const feathers_utils_1 = require("feathers-utils");
const invertedMap = {
    "$gt": "$lte",
    "$gte": "$lt",
    "$lt": "$gte",
    "$lte": "$gt",
    "$in": "$nin",
    "$nin": "$in",
    "$ne": (prop) => {
        return prop["$ne"];
    }
};
const supportedOperators = Object.keys(invertedMap);
const invertedProp = (prop, name) => {
    const map = invertedMap[name];
    if (typeof map === "string") {
        return { [map]: prop[name] };
    }
    else if (typeof map === "function") {
        return map(prop);
    }
};
const convertRuleToQuery = (rule, options) => {
    const { conditions, inverted } = rule;
    if (!conditions) {
        if (inverted && (options === null || options === void 0 ? void 0 : options.actionOnForbidden)) {
            options.actionOnForbidden();
        }
        return undefined;
    }
    if (inverted) {
        const newConditions = {};
        for (const prop in conditions) {
            if (isPlainObject_1.default(conditions[prop])) {
                const obj = conditions[prop];
                for (const name in obj) {
                    if (!supportedOperators.includes(name)) {
                        console.error(`CASL: not supported property: ${name}`);
                        continue;
                    }
                    newConditions[prop] = invertedProp(obj, name);
                }
            }
            else {
                newConditions[prop] = { $ne: conditions[prop] };
            }
        }
        return newConditions;
    }
    else {
        return conditions;
    }
};
exports.convertRuleToQuery = convertRuleToQuery;
const getConditionalQueryFor = (ability, method, subjectType, options) => {
    options = options || {};
    options.actionOnForbidden = options.actionOnForbidden || (() => {
        throw new errors_1.Forbidden("You're not allowed to make this request");
    });
    const rules = ability.rulesFor(method, subjectType).reverse();
    let query = {};
    for (let i = 0; i < rules.length; i++) {
        const rule = rules[i];
        const currentQuery = exports.convertRuleToQuery(rule, options);
        if (currentQuery === undefined) {
            query = {};
        }
        else {
            query = feathers_utils_1.mergeQuery(query, currentQuery, {
                defaultHandle: "combine"
            });
        }
    }
    return query;
};
exports.default = getConditionalQueryFor;
